/*---------------------------------------------------------------------------*
 *
 * Project
 *
 * $RCSfile: fXPlayMidi.c,v $
 * $Date: 1998/04/18 17:01:55 $
 * $Author: klmcw $
 * $Revision: 1.11 $
 *
 *---------------------------------------------------------------------------*
 * $Log: fXPlayMidi.c,v $
 * Revision 1.11  1998/04/18 17:01:55  klmcw
 * Get rid of the rest of the fmAlbum menu callbacks.
 * The events are processed explicitly now (for consistency).
 *
 * Revision 1.10  1997/09/14 01:32:24  klmcw
 * Added -a, -A, -p, -P command line options.
 * Added mnPlayers and callback to do forms loop.
 * Updated Usage message.
 * Created fXPlayMidiRedisplayForm to work some X magic -- oooo, voodoo :)
 *
 * Revision 1.9  1997/04/26 11:40:13  klmcw
 * Added fmAbout.
 *
 * Revision 1.8  1997/04/26 08:23:22  klmcw
 * Revision for libforms 0.86
 * Set browser font size/style to something sane -- finally!
 * Removed support for the Device option.
 * Added support for ".au" files.
 *
 * Revision 1.7  1996/04/21 18:35:59  klmcw
 * Make Window and Icon title include filename during play mode.
 *
 * Revision 1.6  1996/04/19 13:12:42  klmcw
 * Initial port to libforms.so.0.81
 *
 * Revision 1.5  1996/03/19 13:56:24  klmcw
 * Added -b and -v command line switches.
 * Added printing of usage message.
 *
 * Revision 1.4  1996/03/09 08:22:10  klmcw
 * Added firsttime special case to prompt for options when they don't exist yet.
 * Added code to save album upon exit if it has changed.
 * Also, fixed problem with albumLineNumRandom() in albumPlay() if browser is empty.
 * Was causing floating point exception.
 *
 * Revision 1.3  1996/03/02 20:00:10  klmcw
 * Added #includes per -Wall suggestions -- better lint than lint!
 *
 * Revision 1.2  1996/03/02 19:39:08  klmcw
 * Added call to fl_set_atclose() to trap X WM_DESTROY_WINDOW message.
 *
 * Revision 1.1  1996/02/26 02:23:08  klmcw
 * Initial revision
 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/

#include <getopt.h>
#include <limits.h>
#include <memory.h>
#include <stdio.h>

#include "fXPlayMidi.h"
#include "about.h"
#include "album.h"
#include "bldabms.h"
#include "file.h"
#include "mnfile.h"
#include "mnplayers.h"
#include "options.h"
#include "version.h"

#define RangeOK(c1,c,a) ((c1 >= c) && (c1 <= (c + a)))

typedef enum {FXBLD, FXGUI, FXVER} fxMODE;

/*---------------------------------------------------------------------------*/

static int fXPlayMidiPrintVersion(XPMApp_t *);
static int fXPlayMidiUsage(XPMApp_t *);

/*---------------------------------------------------------------------------*/

fxMODE fxMode = FXGUI;

/*---------------------------------------------------------------------------*/
int
main(int argc, char *argv[])
{
  char ch;
  const char *_fxAlbum = NULL, *_fxAlbumFile = NULL;
  enum { fxplNone, fxplHoriz, fxplVert } _fxPlayer = fxplNone;
  int ArgErr = 0;
  XPMApp_t App;
  FL_OBJECT *obj;

  App.Name = "fXPlayMidi";

  while((ch = getopt(argc, argv, "a:A:bpPv")) != -1)
    switch(ch) {
    case 'a': _fxAlbumFile = optarg; break;
    case 'A': _fxAlbum     = optarg; break;
    case 'b': fxMode = FXBLD;        break;
    case 'p': _fxPlayer = fxplHoriz; break;
    case 'P': _fxPlayer = fxplVert;  break;
    case 'v': fxMode = FXVER;        break;
    default:  ArgErr = 1;            break;
    }

  if(ArgErr || ((fxMode == FXBLD) && (optind >= argc)))
    return(fXPlayMidiUsage(&App));

  switch(fxMode) {
  case FXBLD: return(bldabms(argc, argv, optind)); break;
  case FXGUI: break;
  case FXVER: return(fXPlayMidiPrintVersion(&App)); break;
  }

  if(optionsRead(&App) < 0) return(1);

  fl_initialize(&argc, argv, App.Name, 0, 0);
  App.About   = create_form_fmAbout();
  App.Album   = create_form_fmAlbum();
  App.FileSet = create_form_fmFileSettings();
  App.Options = create_form_fmOptions();
  App.HPlayer = create_form_fmHPlayer();
  App.VPlayer = create_form_fmVPlayer();

  /* form initialization code */
  App.fmFirst = NULL;
  App.About  ->fmAbout       ->u_vdata = (void *)&App;
  App.Album  ->fmAlbum       ->u_vdata = (void *)&App;
  App.FileSet->fmFileSettings->u_vdata = (void *)&App;
  App.Options->fmOptions     ->u_vdata = (void *)&App;
  App.HPlayer->fmHPlayer     ->u_vdata = (void *)&App;
  App.VPlayer->fmVPlayer     ->u_vdata = (void *)&App;

  /* Create Window(s) for use later. */
  fl_prepare_form_window(App.Album  ->fmAlbum  ,
			 FL_PLACE_CENTER, FL_FULLBORDER,
			 App.Name);
  fl_prepare_form_window(App.HPlayer->fmHPlayer,
			 FL_PLACE_CENTER, FL_FULLBORDER,
			 App.Name);
  fl_prepare_form_window(App.VPlayer->fmVPlayer,
			 FL_PLACE_CENTER, FL_FULLBORDER,
			 App.Name);

  fl_set_app_mainform(App.Album->fmAlbum);

  fl_set_form_atclose(App.About  ->fmAbout       , fmAbout_atClose       , &App);
  fl_set_form_atclose(App.FileSet->fmFileSettings, fmFileSettings_atClose, &App);
  fl_set_form_atclose(App.Options->fmOptions     , fmOptions_atClose     , &App);

  fmHPlayerDefault(&App);
  fmVPlayerDefault(&App);

  mnfileCreate(&App);
  mnplayersCreate(&App);

  if(_fxAlbumFile != NULL) albumReadPath(&App, _fxAlbumFile);
  else {
    if(_fxAlbum == NULL) _fxAlbum = optionsAlbumGet();

    albumRead(&App, _fxAlbum);
  }

  fl_set_input(App.Album->inAlbum, albumCurrentGet());

  /* If first time, prompt for options. */
  if(optionsFirstTime()) {
    /* Cancel NOT allowed 1st time through. */
    fl_deactivate_object(App.Options->btOptionsCancel);

    mnAppOptions_cb(App.Album->mnAppOptions, 0L);

    while(1) {
      XEvent xevent;
      const XEvent *xp;
      const XButtonReleasedEvent *bev;
      FL_OBJECT
	*OKbp = App.Options->btOptionsOk;

      obj = fl_check_forms();

      xp = fl_last_event();
      if(xp != NULL)
	if(xp->type == ButtonRelease) {
	  bev = (XButtonReleasedEvent *)xp;

	  /* I HATE THIS! But, check to see if Ok was hit. */
	  if(RangeOK(bev->x, OKbp->x, OKbp->w) && RangeOK(bev->y, OKbp->y, OKbp->h))
	    break;
	}

      if(obj == FL_EVENT) fl_XNextEvent(&xevent);
    }
  }
  fl_activate_object(App.Options->btOptionsCancel);

  /* show the first form */
  App.fmFirst = App.Album->fmAlbum;
  if     (_fxPlayer == fxplHoriz) App.fmFirst = App.HPlayer->fmHPlayer;
  else if(_fxPlayer == fxplVert ) App.fmFirst = App.VPlayer->fmVPlayer;

  fl_show_form_window(App.fmFirst);

  while((obj = fl_do_forms()) != NULL)
    if(obj == FL_EVENT) {
      XEvent xevent;
      fl_XNextEvent(&xevent);
    } else if(obj == App.Album->mnFile)
      mnfileProcMenuItem(obj);
    else if(obj == App.Album->mnAbout)
      mnAbout_cb(obj, 0L);
    else if(obj == App.Album->mnAppOptions)
      mnAppOptions_cb(obj, 0L);
    else if(obj == App.Album->mnPlayers)
      mnplayersProcMenuItem(obj);

  return 0;
}
/*---------------------------------------------------------------------------*/
/* This function is for the 3 main views.                                    */
/* These forms are put up using the fl_prepare.. functions so that their     */
/* Window structs are available to XSetWMIconName() in filePlay_SetState().  */
/* See! There _IS_ madness to this method :)                                 */
/*---------------------------------------------------------------------------*/
void
fXPlayMidiRedisplayForm(FL_FORM* fmThis, int firstTime)
{
  XWindowAttributes xwAtt;

  /* DO NOT CALL fl_winshow() IF WINDOW IS VIEWABLE!! */
  XGetWindowAttributes(fl_get_display(), fmThis->window, &xwAtt);
  if(xwAtt.map_state == IsViewable) return;

  if(!firstTime) {
    fl_winshow(fmThis->window);
    fl_redraw_form(fmThis);
  }

  fl_show_form_window(fmThis);
  XFlush(fl_get_display());
}
/*---------------------------------------------------------------------------*/
static int
fXPlayMidiPrintVersion(XPMApp_t *App)
{
  fprintf(stderr, "%s %s\n", App->Name, fXPlayMidiVersion());

  return(0);
}
/*---------------------------------------------------------------------------*/
#include <ctype.h>

const char *
fXPlayMidiVersion(void)
{
  static int ItsThere = 0;
  static char VerStr[80];

  if(!ItsThere) {
    char *cp;
    strcpy(VerStr, fxpmREV);

    for(cp = VerStr; cp && *cp && !isdigit(*cp) && (*cp != '.'); cp++);
    strcpy(VerStr, cp);

    for(cp = VerStr; cp && *cp && (isdigit(*cp) || (*cp == '.')); cp++);
    *cp = '\0';

    ItsThere++;
  }

  return(VerStr);
}
/*---------------------------------------------------------------------------*/
static int
fXPlayMidiUsage(XPMApp_t *App)
{
  (void)fXPlayMidiPrintVersion(App);

  fprintf(stderr,
	  "%s [-a fullFilePath] [-A Album] [-b Directory [...]] [-p] [-P] [-v]\n\n"
	  "        -a allows you to specify an album \"file\" with which to start.\n"
	  "\n"
	  "        -A allows you to specify the name of an album with which to start.\n"
	  "\n"
	  "        -b will build albums for you based on the Directory(s)\n"
	  "        you pass with it. NOTE: You MUST supply at least 1 Directory!\n"
	  "\n"
	  "        -p starts in the Horizontal Player.\n"
	  "\n"
	  "        -P starts in the Vertical Player.\n"
	  "\n"
	  "        -v prints the version and exits.\n"
	  "\n"
	  "        Without any options %s enters its full size GUI mode.\n",
	  App->Name, App->Name);

  return(1);
}
/*---------------------------------------------------------------------------*/
